unit Diag2d;

{
   The following is copyrighted material.

   (c) 1993, 1995 by Michael Leahy
                     BOOKUP Corp.
                     2763 Kensington Place West
                     Columbus, OH  43202-2355
                     U.S.A.

   (800) 949-5445  USA and Canada
   (614) 263-7219  International
   (614) 262-9788  Fax


   Version 1.0b Sep 21, 1995.
   Some changes for Delphi 2.0 added by A. Valverde
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Clipbrd,

  {$IFDEF DEBUG}
  Utils,               { TellTheUser();  Assert(); }
  {$ENDIF}

  DiagType,
  Diagram;



type
  TChessDiagram2D = class(TChessDiagram)
  private
    { Private declarations }

  protected
    { Protected declarations }
    fLastX,
    fLastY,
    fTargetX,
    fTargetY,
    fOriginalX,
    fOriginalY : Integer;

    f2DPieces : array [1..24] of TBitMap;
    fBigOffScreen,
    fSingleSquareOffScreen : TBitMap;


    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState;
                        X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton;
                      Shift: TShiftState;
                      X, Y: Integer); override;

    procedure DrawPiece (PieceNumber : PieceType; Hor, Ver : Integer); override;
    procedure ConstrictCoordinates (var X, Y : Integer); override;
    procedure StartAnimation (PieceNumber : PieceType;
                              theFromSquare,
                              theToSquare : SquareType); override;

  public


    { Public declarations }
    DarkSquaresColor  : TColor;
    LightSquaresColor : TColor;

    constructor Create (aOwner : TComponent); override;
    destructor Destroy; override;

    function WhichSquareIs (MouseX, MouseY : Integer) : SquareType; override;
    procedure DrawPieceOnSquare (PieceNumber : PieceType; theSquare : SquareType); override;
    procedure RefreshSquare (theSquare : SquareType); override;
    procedure Animate; override;
    procedure StopAnimation; override;


  published
    { Published declarations }
    property OnMouseDown;
    property OnMouseUp;
    procedure GetCanvas(var canv:TCanvas);
    procedure SaveDiagramToFile(st:string);
    procedure CopyDiagramToClipBoard;
    function CopyDiagramToBitmap: TBitmap;
  end;

procedure Register;

{This is from Andres Valverde, inicialice this variables in your code}
//  var DarkSquaresColor,LightSquaresColor:TColor;

{======================================================================}
implementation


{$R diag2d.res}



const
  kSquareWidth = 38;    { These figures must be EVEN numbers for dragging. }
  kSquareHeight = 38;


function Min (A, B : LongInt) : LongInt;
begin
  if (A < B)
    then Min := A
    else Min := B;
end;



procedure Register;
begin
  RegisterComponents('Samples', [TChessDiagram2D]);
end;


constructor TChessDiagram2D.Create (aOwner : TComponent);
var
  BitMap : Integer;

  BMPFileName : String [40];

  {BMPResourceName : array[0..19] of Char;}
  bmpresourcename:string[255];
  BMPResourceNumber : array[0..2] of Char;

begin
  inherited Create (aOwner);

  fVersion := '1.0';

  fLastX := 0;
  fLastY := 0;

  Width := 8 * kSquareWidth;
  Height := 8 * kSquareHeight;

  SetBounds (0, 0, Width, Height);


  fBigOffScreen := TBitMap.Create;

  fSingleSquareOffScreen := TBitMap.Create;

  for Bitmap := 1 to 24 do
    begin


 (*  {piece to piece method, Andres Valverde}
      f2DPieces [BitMap] := TBitMap.Create;
      Str (Bitmap:0, BMPFileName);
      BMPFileName :='e:\ectool30\piezas2\piece'+BMPFileName + '.BMP';
      f2DPieces [BitMap].LoadFromFile(BMPFileName);
  *)


  {Changed from 16 bits version A.Valverde}

      try
      f2dpieces[bitmap]:=tbitmap.create;
      bmpresourcename:='PIECE'+inttostr(bitmap);
      f2dpieces[bitmap].LoadFromResourceName(Hinstance,BMPresourcename);
      except
       messagedlg('Error loading '+bmpresourcename,mterror,[mbok],0);
       application.terminate;
      end;

    end;


  fBigOffscreen.Width := 8 * kSquareWidth;
  fBigOffscreen.Height := 8 * kSquareHeight;

  fSingleSquareOffscreen.Width := kSquareWidth;
  fSingleSquareOffscreen.Height := kSquareHeight;

  DarkSquaresColor  :=  clbtnshadow;            //  $00808000;  { green }
  LightSquaresColor :=  clbtnhighlight;          // $00FFFFFF;  { white }
end;


destructor TChessDiagram2D.Destroy;
var
  BitMap : Integer;

begin
  fBigOffScreen.Free;
  fSingleSquareOffScreen.Free;

  for BitMap := 1 to 24 do
    f2DPieces[BitMap].Free;

  inherited Destroy;
end;



procedure TChessDiagram2D.Paint;
var
  Rank, Column : Integer;
  theSquare : SquareType;
  thePieceToDraw : PieceType;

begin
{   with Canvas do
    begin
      Pen.Color := clRed;
      Brush.Color := clRed;
      FillRect (Rect (0, 0, Width, Height));
    end; }


  StopAnimation;

  with Canvas do
    begin
      for Rank := 0 to 7 do
        for Column := 0 to 7 do
          begin
            theSquare := Rank * 10 + Column;

            if (fChessPosition <> nil)
              then thePieceToDraw := fChessPosition.PieceOnSquare (theSquare)
              else thePieceToDraw := kEmptySquare;

            DrawPieceOnSquare (thePieceToDraw, theSquare);
          end;
    end;
end;



procedure TChessDiagram2D.MouseDown(Button: TMouseButton;
                                 Shift: TShiftState;
                                 X, Y: Integer);
var
  theFromSquare : SquareType;
  Rank, Column : Integer;
  Success : Boolean;

begin


  inherited MouseDown(Button, Shift, X, Y);

 {Andres} if fdraggingapiece then exit;{end Andres}

  StopAnimation;

  fItsChessMove.FromSquare := WhichSquareIs (X, Y);
  fItsChessMove.ToSquare := fItsChessMove.FromSquare;
  fItsChessMove.PromotionPiece := kEmptySquare;
  fItsChessMove.Diacritic := 1;

  fPieceBeingDragged := fChessPosition.PieceOnSquare (fItsChessMove.FromSquare);

    { If the user clicked down on an empty square then }
    { bail out. }
  if (fPieceBeingDragged = kEmptySquare)
    then Exit;


    { Copy the entire diagram from the screen to the offscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     Self.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 89010);
  {$ENDIF}



  { BigOffScreen.SaveToFile ('BIGOFF.BMP');    }


    { TEST CODE Wipe out the screen. }
  { Self.Canvas.FillRect (Rect (0, 0,
                              Width,
                              Height));   }

    { TEST CODE Copy the the offscreen back to the screen. }
    { *API* }
  { Success := BitBlt (Self.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     BigOffScreen.Canvas.Handle,
                     0, 0,
                     SRCCOPY);  }

  {$IFDEF DEBUG}
  { Assert (Success, 89020);   }
  {$ENDIF}


  if fWhiteIsAtBottom
    then theFromSquare := fItsChessMove.FromSquare
    else theFromSquare := 77 - fItsChessMove.FromSquare;

    { Calculate the location of this piece in the BigOffscreen. }
  Rank := theFromSquare div 10;
  Column := theFromSquare - (Rank * 10);

 { Wipe out the piece by copying a blank square there. }
 { Draw a blank square in the BigOffscreen. }

  if SquareIsLight (theFromSquare)
    then fBigOffScreen.Canvas.Brush.Color := LightSquaresColor
    else fBigOffScreen.Canvas.Brush.Color := DarkSquaresColor;

    { Keep track of the last location of the whole piece. }
  fLastX := Column * kSquareWidth;
  fLastY := (7 - Rank) * kSquareHeight;

  with fBigOffScreen.Canvas do
    begin
      FillRect (Rect (fLastX,
                      fLastY,
                      fLastX + kSquareWidth,
                      fLastY + kSquareHeight));
    end;

  { MessageBeep (MB_ICONEXCLAMATION); }    { *API* }

  fDraggingAPiece := True;
end;



procedure TChessDiagram2D.MouseMove(Shift: TShiftState;
                                 X, Y: Integer);
var
  X1, Y1, X2, Y2 : Integer;
  BigX1, BigY1, BigX2, BigY2 : Integer;
  Success : Boolean;

begin
  inherited MouseMove(Shift, X, Y);

    { If we're not dragging then exit. }
  if not fDraggingAPiece
    then Exit;

    { If the mouse has not moved then exit. }
  if (X = fLastX) and
     (Y = fLastY)
    then Exit;


  ConstrictCoordinates (X, Y);

  X1 := X - kSquareWidth div 2;
  Y1 := Y - kSquareHeight div 2;
  X2 := X1 + kSquareWidth - 1;
  Y2 := Y1 + kSquareHeight - 1;


    { Copy the area from the BigOffscreen (where we're about }
    { to draw a piece) to the SingleSquareOffscreen. }
    { *API* }
  Success := BitBlt (fSingleSquareOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffScreen.Canvas.Handle,
                     X1, Y1,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 90010);
  {$ENDIF}


    { Copy the piece mask to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     X1, Y1,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged + 12].Canvas.Handle,
                     0, 0,
                     SRCAND);

  {$IFDEF DEBUG}
  Assert (Success, 90020);
  {$ENDIF}


    { Copy the piece to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     X1, Y1,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged].Canvas.Handle,
                     0, 0,
                     SRCPAINT);

  {$IFDEF DEBUG}
  Assert (Success, 90030);
  {$ENDIF}


        { Calculate the intersection of the last rectangle and }
        { the current rectangle the cursor is hovering over. }
        { The hope is that they overlap and allow the opimization of }
        { transferring less total landscape to the screen in a }
        { single call. }
  if (fLastX < X1)
    then BigX1 := fLastX
    else BigX1 := X1;

  if (fLastX + kSquareWidth > X2)
    then BigX2 := fLastX + kSquareWidth
    else BigX2 := X2;

  if (fLastY < Y1)
    then BigY1 := fLastY
    else BigY1 := Y1;

  if (fLastY + kSquareHeight > Y2)
    then BigY2 := fLastY + kSquareHeight
    else BigY2 := Y2;


    { Copy all of the parts of the offscreen that we need }
    { updated (the big rect) back to the screen. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     BigX1, BigY1,
                     BigX2 - BigX1 {+ 1},
                     BigY2 - BigY1 {+ 1},
                     fBigOffScreen.Canvas.Handle,
                     BigX1, BigY1,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 90040);
  {$ENDIF}


    { Fix the damage we did to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     X1, Y1,
                     kSquareWidth,
                     kSquareHeight,
                     fSingleSquareOffscreen.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 90050);
  {$ENDIF}


  fLastX := X1;
  fLastY := Y1;
end;



procedure TChessDiagram2D.MouseUp(Button: TMouseButton;
                               Shift: TShiftState;
                               X, Y: Integer);
var
  Success : Boolean;

begin
  if not fDraggingAPiece
    then Exit;

    { Stop dragging. }
  fDraggingAPiece := False;

    { Erase the piece from the screen by copying }
    { that portion of the BigOffscreen to the screen. }
    { Users of this component will handle mouseups and }
    { are responsible for drawing the piece in the }
    { proper place. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     fLastX, fLastY,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffscreen.Canvas.Handle,
                     fLastX, fLastY,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 91010);
  {$ENDIF}

    { In which square did the user drop the piece? }
  fItsChessMove.ToSquare := WhichSquareIs (fLastX + kSquareWidth div 2,
                                           fLastY + kSquareHeight div 2);

    { If a white pawn has been dragged to the 8th rank then promote. }
  if (fPieceBeingDragged = kWhitePawn) and
     (fItsChessMove.ToSquare > 69)
    then fItsChessMove.PromotionPiece := fPromotionPiece;

    { If a black pawn has been dragged to the 8th rank then promote. }
    { The '+6' is to convert from White to Black. }
  if (fPieceBeingDragged = kBlackPawn) and
     (fItsChessMove.ToSquare < 8)
    then fItsChessMove.PromotionPiece := fPromotionPiece + 6;

    { TEST CODE }
  { RefreshSquare (fItsChessMove.FromSquare);   }


//  fChessPosition.PutPieceOnSquare(kEmptySquare, fItsChessMove.FromSquare);
//  fChessPosition.PutPieceOnSquare(fPieceBeingDragged, fItsChessMove.ToSquare);
//  DrawPieceOnSquare(fPieceBeingDragged, fItsChessMove.ToSquare);


    { Let the user's hooked procedure run, too. }
  inherited MouseUp(Button, Shift, X, Y);
end;



procedure TChessDiagram2D.StartAnimation (PieceNumber : PieceType;
                                       theFromSquare,
                                       theToSquare : SquareType);
var
  Success : Boolean;
  Rank, Column,
  TargetRank, TargetColumn : Integer;

begin
  inherited StartAnimation (PieceNumber,
                            theFromSquare,
                            theToSquare);

  fPieceBeingDragged := PieceNumber;

    { Copy the entire diagram from the screen to the offscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     Self.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 95010);
  {$ENDIF}

    { Skew the square to the other side of the board if the black }
    { pieces are at the bottom of the diagram. }
  if not fWhiteIsAtBottom
    then
      begin
        theFromSquare := 77 - theFromSquare;
        theToSquare := 77 - theToSquare;
      end;


  Rank := theFromSquare div 10;
  Column := theFromSquare - (Rank * 10);

  TargetRank := theToSquare div 10;
  TargetColumn := theToSquare - (TargetRank * 10);


    { Keep track of the last location of the whole piece. }
  fLastX := Column * kSquareWidth;
  fLastY := (7 - Rank) * kSquareHeight;

    { Keep track of the target location for the piece. }
  fTargetX := TargetColumn * kSquareWidth;
  fTargetY := (7 - TargetRank) * kSquareHeight;

  fOriginalX := fLastX;
  fOriginalY := fLastY;


    { Wipe out the piece by copying a blank square there. }
    { Draw a blank square in the BigOffscreen. }
  if SquareIsLight (theFromSquare)
    then fBigOffScreen.Canvas.Brush.Color := LightSquaresColor
    else fBigOffScreen.Canvas.Brush.Color := DarkSquaresColor;

  with fBigOffScreen.Canvas do
    begin
      FillRect (Rect (fLastX,
                      fLastY,
                      fLastX + kSquareWidth,
                      fLastY + kSquareHeight));
    end;


  fAnimatingAMove := True;
end;



procedure TChessDiagram2D.Animate;
var
  Success : Boolean;
  XDistance,
  YDistance,
  DistanceFromTarget,
  DistanceFromSource : Real;
  NudgeAmount,
  XNudgeAmount,
  YNudgeAmount,
  NewX,
  NewY : Integer;
  BigX1, BigX2, BigY1, BigY2 : Integer;

begin
  if not fAnimatingAMove
    then Exit;


  { MessageBeep (MB_ICONEXCLAMATION); }     { *API* }


  XDistance := Abs(fLastX - fTargetX);
  YDistance := Abs(fLastY - fTargetY);


    { Test Code }
  if (XDistance > 7 * kSquareWidth) or
     (YDistance > 7 * kSquareHeight)
    then XDistance := XDistance;
    { End test code }

  DistanceFromTarget := Sqrt ((XDistance * XDistance) +
                              (YDistance * YDistance));

  XDistance := (fLastX - fOriginalX);
  YDistance := (fLastY - fOriginalY);

  DistanceFromSource := Sqrt ((XDistance * XDistance) +
                              (YDistance * YDistance));

    { If the moving piece is within 6 pixels of where it started or }
    { within 6 of where it will end then move }
    { 1 pixel at a time slowly.  Otherwise nudge it at least 3 pixels }
    { but no more than 5 pixels quickly. }
  if (DistanceFromTarget <= 6) or
     (DistanceFromSource <= 6)
    then
      begin
          { We're close to the original location or the target so }
          { nudge it just one pixel. }
        NudgeAmount := 4;

        {TimeAtLastNudge := gxGetTime;

        { Wait at least 10 ticks per delay between single pixel movements. }
            {repeat
            until (gxGetTime > TimeAtLastNudge + 10 * gPrefs.SlidingPieceDelay);

            TimeAtLastNudge := gxGetTime;}
      end
    else
      begin
          { TEST CODE }
        NudgeAmount := 5;

              { Wait until the nudge amount is at least 3 pixels. }
            { repeat
              NudgeAmount := (gxGetTime - TimeAtLastNudge);
            until (NudgeAmount >= 3);  }

              { Limit the nudge to no more than 5 pixels. }
            { if (NudgeAmount > 5)
              then NudgeAmount := 5;  }

              { Wait at least 5 ticks per delay between nudges. }
            { repeat
            until (gxGetTime > TimeAtLastNudge + 5 * gPrefs.SlidingPieceDelay);

            TimeAtLastNudge := gxGetTime;   }
      end;


      { Calculate the coordinates where we're about to draw the piece. }
  if (fLastX <= fTargetX)
    then
      begin
        { Nudge as little as necessary.  Otherwise you may }
        { overnudge! }
        XNudgeAmount := Min (NudgeAmount, Abs (fLastX - fTargetX));

        NewX := fLastX + XNudgeAmount;
      end;

  if (fLastX > fTargetX)
    then
      begin
        XNudgeAmount := Min (NudgeAmount, Abs (fLastX - fTargetX));

        NewX := fLastX - XNudgeAmount;
      end;

  if (fLastY <= fTargetY)
    then
      begin
          { Nudge as little as necessary.  Otherwise you may }
          { overnudge! }
        YNudgeAmount := Min (NudgeAmount, Abs (fLastY - fTargetY));

        NewY := fLastY + YNudgeAmount;
      end;

  if (fLastY > fTargetY)
    then
      begin
        YNudgeAmount := Min (NudgeAmount, Abs (flastY - fTargetY));

        NewY := fLastY - YNudgeAmount;
      end;


    { Copy the area from the BigOffscreen (where we're about }
    { to draw a piece) to the SingleSquareOffscreen. }
    { *API* }
  Success := BitBlt (fSingleSquareOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffScreen.Canvas.Handle,
                     NewX, NewY,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 96010);
  {$ENDIF}


    { Copy the piece mask to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     NewX, NewY,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged + 12].Canvas.Handle,
                     0, 0,
                     SRCAND);

  {$IFDEF DEBUG}
  Assert (Success, 96020);
  {$ENDIF}

  {$IFDEF DEBUG}
  Assert ((NewX >= 0) and
          (NewX <= 8 * kSquareWidth) and
          (NewY >= 0) and
          (NewY <= 8 * kSquareHeight), 96021);
  {$ENDIF}

    { Copy the piece to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     NewX, NewY,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged].Canvas.Handle,
                     0, 0,
                     SRCPAINT);

  {$IFDEF DEBUG}
  Assert (Success, 96030);
  {$ENDIF}

        { Calculate the intersection of the last rectangle and }
        { the current rectangle the cursor is hovering over. }
        { The hope is that they overlap and allow the opimization of }
        { transferring less total landscape to the screen in a }
        { single call. }
  if (fLastX < NewX)
    then BigX1 := fLastX
    else BigX1 := NewX;

  if (fLastX > NewX)
    then BigX2 := fLastX + kSquareWidth
    else BigX2 := NewX + kSquareWidth;

  if (fLastY < NewY)
    then BigY1 := fLastY
    else BigY1 := NewY;

  if (fLastY > NewY)
    then BigY2 := fLastY + kSquareHeight
    else BigY2 := NewY + kSquareHeight;


    { Copy all of the parts of the offscreen that we need }
    { updated (the big rect) back to the screen. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     BigX1, BigY1,
                     BigX2 - BigX1,
                     BigY2 - BigY1,
                     fBigOffScreen.Canvas.Handle,
                     BigX1, BigY1,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 96040);
  {$ENDIF}


    { Fix the damage we did to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     NewX, NewY,
                     kSquareWidth,
                     kSquareHeight,
                     fSingleSquareOffscreen.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 96050);
  {$ENDIF}


    { Keep track of the 'new' last location of the piece. }
  fLastX := NewX;
  fLastY := NewY;


    { If the piece has arrived at the target location then stop. }
  if (fLastX = fTargetX) and
     (fLastY = fTargetY)
    then StopAnimation;


  { MessageBeep (MB_ICONEXCLAMATION); }     { *API* }
end;



procedure TChessDiagram2D.StopAnimation;
var
  Success : Boolean;

begin
  if not fAnimatingAMove
    then Exit;

  fAnimatingAMove := False;

    { Erase the area where the piece is currently by }
    { copying its area from the big offscreen. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     fLastX, fLastY,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffScreen.Canvas.Handle,
                     fLastX, fLastY,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 97005);
  {$ENDIF}



  DrawPieceOnSquare (fPieceBeingDragged,
                     fTargetSquare);

  Exit;

    { TEST CODE }

    { Copy the area from the BigOffscreen (where we're about }
    { to draw a piece) to the SingleSquareOffscreen. }
    { *API* }
  Success := BitBlt (fSingleSquareOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffScreen.Canvas.Handle,
                     fTargetX, fTargetY,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 97010);
  {$ENDIF}


    { Copy the piece mask to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     fTargetX, fTargetY,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged + 12].Canvas.Handle,
                     0, 0,
                     SRCAND);

  {$IFDEF DEBUG}
  Assert (Success, 97020);
  {$ENDIF}

    { Copy the piece to the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     fTargetX, fTargetY,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [fPieceBeingDragged].Canvas.Handle,
                     0, 0,
                     SRCPAINT);

  {$IFDEF DEBUG}
  Assert (Success, 97030);
  {$ENDIF}

    { Copy the piece from the offscreen back to the screen. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     fTargetX, fTargetY,
                     kSquareWidth,
                     kSquareHeight,
                     fBigOffScreen.Canvas.Handle,
                     fTargetX, fTargetY,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 97040);
  {$ENDIF}


    { Fix the damage we did to the BigOffscreen. }
    { This may not even be necessary because }
    { dragging and animating will make a new copy }
    { of the screen in the BigOffscreen. }
    { *API* }
  Success := BitBlt (fBigOffScreen.Canvas.Handle,
                     fTargetX, fTargetY,
                     kSquareWidth,
                     kSquareHeight,
                     fSingleSquareOffscreen.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  {$IFDEF DEBUG}
  Assert (Success, 97050);
  {$ENDIF}
end;



procedure TChessDiagram2D.RefreshSquare (theSquare : SquareType);
var
  thePieceToDraw : PieceType;

begin
  thePieceToDraw := fChessPosition.PieceOnSquare (theSquare);

  DrawPieceOnSquare (thePieceToDraw, theSquare);
end;



procedure TChessDiagram2D.DrawPiece (PieceNumber : PieceType; Hor, Ver : Integer);
var
  Success : Boolean;

begin
  if (PieceNumber = kEmptySquare)
    then Exit;

    { Put down the mask. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     Hor, Ver,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [PieceNumber + 12].Canvas.Handle,
                     0, 0,
                     SRCAND);

  {$IFDEF DEBUG}
  Assert (Success, 95010);
  {$ENDIF}


    { Put down the piece on top of the mask. }
    { *API* }
  Success := BitBlt (Self.Canvas.Handle,
                     Hor, Ver,
                     kSquareWidth,
                     kSquareHeight,
                     f2DPieces [PieceNumber].Canvas.Handle,
                     0, 0,
                     SRCPAINT);

  {$IFDEF DEBUG}
  Assert (Success, 95010);
  {$ENDIF}
end;



procedure TChessDiagram2D.DrawPieceOnSquare (PieceNumber : PieceType;
                                             theSquare : SquareType);
var
  Rank, Column : Integer;
  Success : Boolean;

begin
  if not fWhiteIsAtBottom
    then theSquare := 77 - theSquare;

    { Calculate where the piece should be drawn. }
  Rank := theSquare div 10;
  Column := theSquare - (Rank * 10);

  if SquareIsLight (theSquare)
    then Canvas.Brush.Color := LightSquaresColor
    else Canvas.Brush.Color := DarkSquaresColor;

  with Canvas do
    FillRect (Rect (Column * kSquareWidth,
                    (7 - Rank) * kSquareHeight,
                    (Column + 1) * kSquareWidth,
                    (8 - Rank) * kSquareHeight));

  DrawPiece (PieceNumber,
             Column * kSquareWidth,
             (7 - Rank) * kSquareHeight);
end;



function TChessDiagram2D.WhichSquareIs (MouseX, MouseY : Integer) : SquareType;
var
  theSquare : SquareType;

begin
  ConstrictCoordinates (MouseX, MouseY);

  theSquare := 10 * (7 - (MouseY div kSquareHeight)) + MouseX div kSquareWidth;

  if not fWhiteIsAtBottom
    then theSquare := 77 - theSquare;

  WhichSquareIs := theSquare;
end;



procedure TChessDiagram2D.ConstrictCoordinates (var X, Y : Integer);
begin
    { If less than half a square from the left edge... }
  if (X < kSquareWidth div 2)
    then X := kSquareWidth div 2;

    { If less than half a square from the right edge... }
  if (X > (8 * kSquareWidth - kSquareWidth div 2))
    then X := (8 * kSquareWidth - kSquareWidth div 2);

    { If less than half a square from the top edge... }
  if (Y < kSquareHeight div 2)
    then Y := kSquareHeight div 2;

    { If less than half a square from the bottom edge... }
  if (Y > (8 * kSquareHeight - kSquareHeight div 2))
    then Y := (8 * kSquareHeight - kSquareHeight div 2);
end;

procedure TChessDiagram2D.GetCanvas(var canv:TCanvas);

begin
  canv:=Canvas;
end;


{Added by A. Valverde}
procedure TChessDiagram2D.SaveDiagramToFile(st:string);

begin

    { Copy the entire diagram from the screen to the offscreen. }
    { *API* }
  BitBlt (fBigOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     Self.Canvas.Handle,
                     0, 0,
                     SRCCOPY);
  fbigoffscreen.savetofile(st);

end;

{Added by M. Rosenboom}
procedure TChessDiagram2D.CopyDiagramToClipboard;

begin

    { Copy the entire diagram from the screen to the offscreen. }
    { *API* }
  BitBlt (fBigOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     Self.Canvas.Handle,
                     0, 0,
                     SRCCOPY);
  Clipboard.Assign(fbigoffscreen);

end;

{Added by M. Rosenboom}
function TChessDiagram2D.CopyDiagramToBitmap: TBitmap;

begin

  { Copy the entire diagram from the screen to the offscreen. }
  { *API* }
  BitBlt (fBigOffScreen.Canvas.Handle,
                     0, 0,
                     kSquareWidth * 8,
                     kSquareHeight * 8,
                     Self.Canvas.Handle,
                     0, 0,
                     SRCCOPY);

  CopyDiagramToBitmap := fBigOffScreen;
end;


begin


end.
